"""The 'eznum' package provides enum types."""
#  MIT Licence
#  Copyright (c) 2024 Asger Jon Vistisen
from __future__ import annotations

from ._auto import auto
from ._eznum_meta import EZnumMeta, EZnum
