"""The 'ezside.windows.bars' package provides convenient implementations of
the status bar and menu bar for the main application window."""
#  MIT Licence
#  Copyright (c) 2024 Asger Jon Vistisen
from __future__ import annotations

from ._menu_bar import MenuBar
from ._status_bar import StatusBar
