"""The 'settings' module centralizes default settings."""
#  MIT Licence
#  Copyright (c) 2024 Asger Jon Vistisen
from __future__ import annotations

from ._default import Default
from ._icons import Icons
