"""This package provides a central registry of the keyboard shortcuts used
by the main application."""
#  MIT Licence
#  Copyright (c) 2024 Asger Jon Vistisen
from __future__ import annotations

from ._get_shortcut import getShortcut
