"""The 'ezside.windows.menus' package provides the menus for the main
application window."""
#  MIT Licence
#  Copyright (c) 2024 Asger Jon Vistisen
from __future__ import annotations

from ._action import Action
from ._abstract_menu import AbstractMenu
from ._files_menu import FilesMenu
from ._edit_menu import EditMenu
from ._help_menu import HelpMenu
from ._debug_menu import DebugMenu
